//   For additional information
//   arXiv Link:
//   Cadi: CMS-EXO-21-002
//   Primary author: Angira Rastogi  (angira.rastogi@cern.ch)
//   October 26, 2021
//

#include <TROOT.h>
#include <TFile.h>
#include <fstream>
#include <iostream>
#include <TH1.h>
#include <TH2.h>
#include <TMath.h>
#include "TLorentzVector.h"
#include "TRandom3.h"

// -------------------------------------------------------------
//  Usage
//  float recopT =  eff(float x, float genpt, float geneta, int pdgid, int tauDM=1, int mother=0, float drmin=1, float nj=2, int dRmin_OR_Nj=1)
//  -------------------------------------------------------------
//
//  A function to return a reconstructed pT for a generator-level lepton, given input parameters:
//   x, genpt, geneta, pdgid, tauDM, drmin, nj, mother, dRmin_OR_Nj
//  These parameters are descibed below:
//
// x  ==> Random number between 0 and 1 with uniform probability.
//       In ROOT, one can use the below function as an example:
//       TRandom3 *g = new TRandom3();
//       x = g->Rndm();
//
// genpt, geneta, pdgid ==> pT, pseudorapidity and flavor of the generator-level lepton
//                        The flavor can be given as absolute value
//                        Allowed values are 11, 13, 15 (where 15 is for a hadronically decaying tau)
//
// tauDM   ==> Decay Mode for hadronic taus: tauDM=1 (1-prong) and tauDM=3 (3-prong).
//             For light leptons, use tauDM=0
//             The DM can be identified at the gen level from the number of charged
//             hadronic daughters of status=2 tau leptons
//
// mother  ==> Information about the mother of the light leptons:
//             mother=0 (for light leptons from bosons or signal particles) and
//             mother=1 (for light leptons from taus)
//
// drmin   ==> Minimum dR between any light lepton pair in the event.
//             dR = sqrt( d_eta^2 + d_phi^2 )
//             If no light lepton pair is found, then set drmin=1
//
// nj      ==> Number of selected gen-level jets
//
// dRmin_OR_Nj ==> Choice of global event parametrization:
//                 dRmin_OR_Nj=0 (for dRmin based parametrization) and
//                 dRmin_OR_Nj=1 (for Nj based parametrization)
//
// ----------------------------------------------------------------------------------
//  Important: By default, we have set tauDM=1, drmin=1, nj=2, mother=0, dRmin_OR_Nj=1 i.e.
//   using 1-prong taus over 3-prong taus, events with no light leptons close to each other upto DR=1.0,
//   events with jet multiplicity nj=2, leptons from bosons or signal particles, and Nj-based global event parametrization.
//  Please use the appropriate values for these paramters as per your use case.
// ----------------------------------------------------------------------------------


float eff(float x, float genpt, float geneta, int pdgid, int tauDM=1, int mother=0, float drmin=1, float nj=2, int dRmin_OR_Nj=1)
{
  
  if(abs(pdgid)!=11 && abs(pdgid)!=13 && abs(pdgid)!=15){
    cout<<"Please enter correct PDG ID of the leptons (+/-11, +/-13 or +/-15)"<<endl;
    return 0;
  }
  if((tauDM!=1 && tauDM!=3) && abs(pdgid)==15){
    cout<<"Please enter correct decay mode of hadronic tau lepton (DM=1 for 1-prong, DM=3 for 3-prong)"<<endl;
    return 0;
  }
  if(mother!=0 && mother!=1){
    cout<<"Please enter correct mother information of the leptons (mother=0 for light leptons from bosons or signal particles, and mother=1 for light leptons from taus)"<<endl;
    return 0;
  }
  if(dRmin_OR_Nj!=0 && dRmin_OR_Nj!=1){
    cout<<"Please enter correct global event parametrization you want to use (dRmin_OR_Nj=0 for dRmin based parametrization and dRmin_OR_Nj=1 for Nj based parametrization)"<<endl;
    return 0;
  }

  TFile *effmap = new TFile("EffMap_ZZ_Incl_2018_Allbinning.root");
  
  float llbarrelpt[24] = {0,5,10,15,20,25,30,35,40,50,60,70,80,100,150,200,250,300,350,400,450,500,750,1000};
  float llendcappt[23] = {0,5,10,15,20,25,30,35,40,50,60,70,80,100,150,200,250,300,350,400,450,500,1000};
  float tau1pbarrelpt[22] = {0,15,20,25,30,35,40,50,60,70,80,100,150,200,250,300,350,400,450,500,750,1000};
  float tau1pendcappt[18] = {0,15,20,25,30,35,40,50,60,70,80,100,150,200,250,350,450,1000};
  float tau3pbarrelpt[22] = {0,15,20,25,30,35,40,50,60,70,80,100,150,200,250,300,350,400,450,500,750,1000};
  float tau3pendcappt[18] = {0,15,20,25,30,35,40,50,60,70,80,100,150,200,250,350,450,1000};
  float llbarrelpt_tauMom[20] = {0,5,10,15,20,25,30,35,40,50,60,70,80,100,150,200,300,400,500,1000};
  float llendcappt_tauMom[19] = {0,5,10,15,20,25,30,35,40,50,60,70,80,100,150,200,300,500,1000};

  TH2F *hist;
  int binN = 0;
  float pt = -1;
  vector<float> effVal, pTVal;
  TString plotname;
  
  if(dRmin_OR_Nj==0){ //DR parametrization
    if(abs(pdgid)==11){ //electron
      if(fabs(geneta)<=1.1 && drmin>0.2){
	if(mother==0){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Electron_barrelpt_BosonMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0]); i++){
	      if(i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i] && genpt<llbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt[i]+llbarrelpt[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Electron_barrelpt_BosonMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0]); i++){
	      if(i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i] && genpt<llbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt[i]+llbarrelpt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Electron_barrelpt_TauMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0]); i++){
	      if(i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i] && genpt<llbarrelpt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt_tauMom[i]+llbarrelpt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Electron_barrelpt_TauMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0]); i++){
	      if(i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i] && genpt<llbarrelpt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt_tauMom[i]+llbarrelpt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
      if((fabs(geneta)>1.1 && fabs(geneta)<=1.6) && drmin>0.2){
	if(mother==0){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Electron_transitionpt_BosonMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Electron_transitionpt_BosonMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Electron_transitionpt_TauMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Electron_transitionpt_TauMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
      if(fabs(geneta)>1.6 && drmin>0.2){
	if(mother==0){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Electron_endcappt_BosonMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Electron_endcappt_BosonMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Electron_endcappt_TauMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Electron_endcappt_TauMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
    }
    
    if(abs(pdgid)==13){ //muon
      if(fabs(geneta)<=1.2 && drmin>0.2){
	if(mother==0){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Muon_barrelpt_BosonMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0]); i++){
	      if(i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i] && genpt<llbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt[i]+llbarrelpt[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Muon_barrelpt_BosonMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0]); i++){
	      if(i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i] && genpt<llbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt[i]+llbarrelpt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Muon_barrelpt_TauMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0]); i++){
	      if(i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i] && genpt<llbarrelpt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt_tauMom[i]+llbarrelpt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Muon_barrelpt_TauMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0]); i++){
	      if(i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i] && genpt<llbarrelpt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt_tauMom[i]+llbarrelpt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
      if(fabs(geneta)>1.2 && drmin>0.2){
	if(mother==0){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Muon_endcappt_BosonMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Muon_endcappt_BosonMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if((drmin>0.2 && drmin<0.4)){
	    plotname = "Muon_endcappt_TauMother_DRLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(drmin>0.4){
	    plotname = "Muon_endcappt_TauMother_DRHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
    }

    if(abs(pdgid)==15){ //tau
      if(tauDM==1){ //1-prong
	if(fabs(geneta)<=1.1){
	  if(drmin>0.2){
	    plotname = "Tau1p_barrelpt_BosonMother_DRIncl_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0]); i++){
	      if(i<sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0])-1){
		if(genpt>=tau1pbarrelpt[i] && genpt<tau1pbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0])-1){
		if(genpt>=tau1pbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pbarrelpt[i]+tau1pbarrelpt[i-1])/2);
	      }
	    }
	  }
	}
	if(fabs(geneta)>1.1 && fabs(geneta)<=1.6){
	  if(drmin>0.2){
	    plotname = "Tau1p_transitionpt_BosonMother_DRIncl_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0]); i++){
	      if(i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i] && genpt<tau1pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pendcappt[i]+tau1pendcappt[i-1])/2);
	      }
	    }
	  }
	}
	if(fabs(geneta)>1.6){
	  if(drmin>0.2){
	    plotname = "Tau1p_endcappt_BosonMother_DRIncl_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0]); i++){
	      if(i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i] && genpt<tau1pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pendcappt[i]+tau1pendcappt[i-1])/2);
	      }
	    }
	  }
	}
      }
      if(tauDM==3){ //3-prong
	if(fabs(geneta)<=1.1){
	  if(drmin>0.2){
	    plotname = "Tau3p_barrelpt_BosonMother_DRIncl_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0]); i++){
	      if(i<sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0])-1){
		if(genpt>=tau3pbarrelpt[i] && genpt<tau3pbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0])-1){
		if(genpt>=tau3pbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pbarrelpt[i]+tau3pbarrelpt[i-1])/2);
	      }
	    }
	  }
	}
	if(fabs(geneta)>1.1 && fabs(geneta)<=1.6){
	  if(drmin>0.2){
	    plotname = "Tau3p_transitionpt_BosonMother_DRIncl_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0]); i++){
	      if(i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i] && genpt<tau3pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pendcappt[i]+tau3pendcappt[i-1])/2);
	      }
	    }
	  }          
	}
	if(fabs(geneta)>1.6){
	  if(drmin>0.2){
	    plotname = "Tau3p_endcappt_BosonMother_DRIncl_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0]); i++){
	      if(i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i] && genpt<tau3pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pendcappt[i]+tau3pendcappt[i-1])/2);
	      }
	    }
	  }      
	}
      }
    }
  }
  
  if(dRmin_OR_Nj==1){ //Nj parametrization
    if(abs(pdgid)==11){ //electron
      if(fabs(geneta)<=1.1 && drmin>0.2){
	if(mother==0){
	  if(nj<2){
	    plotname = "Electron_barrelpt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0]); i++){
	      if(i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i] && genpt<llbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt[i]+llbarrelpt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Electron_barrelpt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0]); i++){
	      if(i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i] && genpt<llbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt[i]+llbarrelpt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if(nj<2){
	    plotname = "Electron_barrelpt_TauMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0]); i++){
	      if(i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i] && genpt<llbarrelpt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt_tauMom[i]+llbarrelpt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Electron_barrelpt_TauMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0]); i++){
	      if(i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i] && genpt<llbarrelpt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt_tauMom[i]+llbarrelpt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
      if((fabs(geneta)>1.1 && fabs(geneta)<=1.6) && drmin>0.2){
	if(mother==0){
	  if(nj<2){
	    plotname = "Electron_transitionpt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Electron_transitionpt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if(nj<2){
	    plotname = "Electron_transitionpt_TauMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Electron_transitionpt_TauMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
      if(fabs(geneta)>1.6 && drmin>0.2){
	if(mother==0){
	  if(nj<2){
	    plotname = "Electron_endcappt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Electron_endcappt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if(nj<2){
	    plotname = "Electron_endcappt_TauMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Electron_endcappt_TauMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
    }
    
    if(abs(pdgid)==13){ //muon
      if(fabs(geneta)<=1.2 && drmin>0.2){
	if(mother==0){
	  if(nj<2){
	    plotname = "Muon_barrelpt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0]); i++){
	      if(i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i] && genpt<llbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt[i]+llbarrelpt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Muon_barrelpt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0]); i++){
	      if(i<sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i] && genpt<llbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt)/sizeof(llbarrelpt[0])-1){
		if(genpt>=llbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt[i]+llbarrelpt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if(nj<2){
	    plotname = "Muon_barrelpt_TauMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0]); i++){
	      if(i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i] && genpt<llbarrelpt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt_tauMom[i]+llbarrelpt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Muon_barrelpt_TauMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0]); i++){
	      if(i<sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i] && genpt<llbarrelpt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llbarrelpt_tauMom)/sizeof(llbarrelpt_tauMom[0])-1){
		if(genpt>=llbarrelpt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llbarrelpt_tauMom[i]+llbarrelpt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
      if(fabs(geneta)>1.2 && drmin>0.2){
	if(mother==0){
	  if(nj<2){
	    plotname = "Muon_endcappt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Muon_endcappt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt)/sizeof(llendcappt[0]); i++){
	      if(i<sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i] && genpt<llendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt)/sizeof(llendcappt[0])-1){
		if(genpt>=llendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt[i]+llendcappt[i-1])/2);
	      }
	    }
	  }
	}
	if(mother==1){
	  if(nj<2){
	    plotname = "Muon_endcappt_TauMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Muon_endcappt_TauMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0]); i++){
	      if(i<sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i] && genpt<llendcappt_tauMom[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(llendcappt_tauMom)/sizeof(llendcappt_tauMom[0])-1){
		if(genpt>=llendcappt_tauMom[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((llendcappt_tauMom[i]+llendcappt_tauMom[i-1])/2);
	      }
	    }
	  }
	}
      }
    }

    if(abs(pdgid)==15){ //tau
      if(tauDM==1){ //1-prong
	if(fabs(geneta)<=1.1){
	  if(nj<2){
	    plotname = "Tau1p_barrelpt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0]); i++){
	      if(i<sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0])-1){
		if(genpt>=tau1pbarrelpt[i] && genpt<tau1pbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0])-1){
		if(genpt>=tau1pbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pbarrelpt[i]+tau1pbarrelpt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Tau1p_barrelpt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0]); i++){
	      if(i<sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0])-1){
		if(genpt>=tau1pbarrelpt[i] && genpt<tau1pbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pbarrelpt)/sizeof(tau1pbarrelpt[0])-1){
		if(genpt>=tau1pbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pbarrelpt[i]+tau1pbarrelpt[i-1])/2);
	      }
	    }
	  }
	}
	if(fabs(geneta)>1.1 && fabs(geneta)<=1.6){
	  if(nj<2){
	    plotname = "Tau1p_transitionpt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0]); i++){
	      if(i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i] && genpt<tau1pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pendcappt[i]+tau1pendcappt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Tau1p_transitionpt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0]); i++){
	      if(i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i] && genpt<tau1pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pendcappt[i]+tau1pendcappt[i-1])/2);
	      }
	    }
	  }
	}
	if(fabs(geneta)>1.6){
	  if(nj<2){
	    plotname = "Tau1p_endcappt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0]); i++){
	      if(i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i] && genpt<tau1pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pendcappt[i]+tau1pendcappt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Tau1p_endcappt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0]); i++){
	      if(i<sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i] && genpt<tau1pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau1pendcappt)/sizeof(tau1pendcappt[0])-1){
		if(genpt>=tau1pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau1pendcappt[i]+tau1pendcappt[i-1])/2);
	      }
	    }
	  }
	}
      }
      if(tauDM==3){ //3-prong
	if(fabs(geneta)<=1.1){
	  if(nj<2){
	    plotname = "Tau3p_barrelpt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0]); i++){
	      if(i<sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0])-1){
		if(genpt>=tau3pbarrelpt[i] && genpt<tau3pbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0])-1){
		if(genpt>=tau3pbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pbarrelpt[i]+tau3pbarrelpt[i-1])/2);
	      }
	    }
	  }
	  if(nj>1){
	    plotname = "Tau3p_barrelpt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0]); i++){
	      if(i<sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0])-1){
		if(genpt>=tau3pbarrelpt[i] && genpt<tau3pbarrelpt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pbarrelpt)/sizeof(tau3pbarrelpt[0])-1){
		if(genpt>=tau3pbarrelpt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pbarrelpt[i]+tau3pbarrelpt[i-1])/2);
	      }
	    }
	  }
	}
	if(fabs(geneta)>1.1 && fabs(geneta)<=1.6){
	  if(nj<2){
	    plotname = "Tau3p_transitionpt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0]); i++){
	      if(i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i] && genpt<tau3pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pendcappt[i]+tau3pendcappt[i-1])/2);
	      }
	    }
	  }          
	  if(nj>1){
	    plotname = "Tau3p_transitionpt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0]); i++){
	      if(i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i] && genpt<tau3pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pendcappt[i]+tau3pendcappt[i-1])/2);
	      }
	    }
	  }          
	}
	if(fabs(geneta)>1.6){
	  if(nj<2){
	    plotname = "Tau3p_endcappt_BosonMother_NjLow_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0]); i++){
	      if(i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i] && genpt<tau3pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pendcappt[i]+tau3pendcappt[i-1])/2);
	      }
	    }
	  }      
	  if(nj>1){
	    plotname = "Tau3p_endcappt_BosonMother_NjHigh_rebin";
	    hist = (TH2F *)effmap->Get(plotname);
	    for(long unsigned int i=0; i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0]); i++){
	      if(i<sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i] && genpt<tau3pendcappt[i+1])
		  binN = i+1;
	      }
	      if(i==sizeof(tau3pendcappt)/sizeof(tau3pendcappt[0])-1){
		if(genpt>=tau3pendcappt[i])
		  binN = i;
	      }
	    }
	    for(int i=1; i<=hist->GetNbinsX(); i++){
	      if(hist->GetBinContent(i,binN)>0){
		effVal.push_back(hist->GetBinContent(i,binN));
		pTVal.push_back((tau3pendcappt[i]+tau3pendcappt[i-1])/2);
	      }
	    }
	  }      
	}
      }
    }
  }
  
  float effsum = 0.00001;
  
  for(int j=0; j<(int)effVal.size(); j++){
    if(x>=effsum && x<(effVal[j]+effsum)){
      pt = pTVal[j];
      break;
    }
    effsum += effVal[j];
  }      

  return pt;
}
